import sys
import os
import numpy as np
import cv2
import json
from collections import defaultdict
import unittest
import torch

sys.path.insert(0, os.path.abspath('')) # Test files from current path rather than installed module
from pymlutil.jsonutil import *

test_config = 'test.yaml'

class Test(unittest.TestCase):
    def test_cmd(self):
        result, _, _ = cmd('ls -la', check=True, timeout=5)
        self.assertEqual(result, 0)

    def test_yaml(self):
        test = ReadDict(test_config)
        assert test is not None
        assert 'test_yaml' in test
        self.assertEqual(test['test_yaml'][0]['zero'], 0)
        self.assertEqual(test['test_yaml'][1]['one'], 1)
        self.assertEqual(test['test_yaml'][2]['two'], 2)

if __name__ == '__main__':
    unittest.main()


import torch
import math
import numpy as np

def GaussianBasis(x, zero=0.0, sigma=0.33):
    return torch.exp(-1*torch.square((x-zero)/(2*sigma*sigma))) # torch.square not supported by torch.onnx

def NormGausBasis(len, i, depth, r=1.0):
        den = 0.0
        num = 0.0
        for j in range(len):
            bias = GaussianBasis(j,depth,r)
            if j==i:
                num=bias
            den = den + bias
        return num/den


# Exponential function from vertex and point
class Exponential():
    def __init__(self,vx=0.0, vy=0.0, px=1.0, py=1.0, power=2.0):
        self.vx = vx
        self.vy = vy
        self.px = px
        self.py = py
        if power < 0:
            raise ValueError('Exponential error power {} must be >= 0'.format(power))
        self.power = power
        if px <= vx:
            raise ValueError('Exponential error px={} must be > vx'.format(px, vx))
        else:
            self.a = (py-vy)/np.power(px-vx,self.power)
    def f(self, x):

        y = np.piecewise(x, 
            [x < self.vx, x > self.px], 
            [self.vy, self.py, lambda x : self.a*pow(x-self.vx,self.power) + self.vy]) 

        return y

def Sigmoid(x, scale = 1.0, offset=0.0, k_exp = 0.1):
    sigmoid = scale/(1.0+np.exp(-1*k_exp*(x-offset)))
    return sigmoid


import sys
import os
import numpy as np
import cv2
import json
from collections import defaultdict
import unittest
import torch

sys.path.insert(0, os.path.abspath('')) # Test files from current path rather than installed module
from pymlutil.functions import *



class Test(unittest.TestCase):
    def test_GaussianBasis(self):
        self.assertEqual(GaussianBasis(torch.tensor(0.0), zero=0.0, sigma=0.33), 1.0)

    def test_GaussianExponential(self):
        vx = 1
        vy = 2
        px = 3
        py = 4
        power = 2
        expf =  Exponential(vx=vx, vy=vy, px=px, py=py, power=power)
        x = np.arange(vx-1, px+1, 0.1)  
        y = expf.f(x)
        assert(y[0] == vy)
        assert(y[-1] == py)

    def test_Sigmoid(self):
        scale = 1.0
        offset = 0.0
        y = Sigmoid(0.0, scale = scale, offset=offset, k_exp = 0.1)
        self.assertAlmostEqual(y,  scale/2.0)


if __name__ == '__main__':
    unittest.main()


import sys
import os
import numpy as np
import cv2
import json
from collections import defaultdict
import unittest
import torch
import tempfile
from filecmp import dircmp

sys.path.insert(0, os.path.abspath('')) # Test files from current path rather than installed module
from pymlutil.s3 import *

test_config = 'test.yaml'

class Test(unittest.TestCase):
    #  PutDir(self, bucket, path, setname)
    def test_PutDir(self):

        parameters = ReadDict(test_config)
        if 's3' not in parameters:
            raise ValueError('s3 not in {}'.format(test_config))

        s3, _, s3def = Connect(parameters['s3']['credentials'])

        set = None
        if parameters['s3']['testset'] in s3def['sets']:
            set = s3def['sets'][parameters['s3']['testset']]
        else:
            raise ValueError('{} {} clone set undefined {}'.format(__file__, __name__, parameters['s3']['testset']))

        destobjpath = set['prefix']
        if destobjpath is not None and len(destobjpath) > 0:
            destobjpath += '/'
        destobjpath += parameters['s3']['objectpath']

        if not (s3.PutDir(set['bucket'], parameters['s3']['srcpath'], destobjpath)):
             raise ValueError('{} {} failed'.format(__file__, __name__))

        #  RemoveObjects(self, bucket, setname=None, pattern='**', recursive=False):
        if not s3.RemoveObjects(set['bucket'], destobjpath, recursive=True):
            raise ValueError('{} {} RemoveObjects({}, {}, recursive={}) failed'.format(__file__, __name__, set['bucket'], destobjpath, recursive=True))

    def test_GitDir(self):

        parameters = ReadDict(test_config)
        if 's3' not in parameters:
            raise ValueError('s3 not in {}'.format(test_config))

        s3, _, s3def = Connect(parameters['s3']['credentials'])

        set = None
        if parameters['s3']['testset'] in s3def['sets']:
            set = s3def['sets'][parameters['s3']['testset']]
        else:
            raise ValueError('{} {} clone set undefined {}'.format(__file__, __name__, parameters['s3']['testset']))

        destobjpath = set['prefix']
        if destobjpath is not None and len(destobjpath) > 0:
            destobjpath += '/'
        destobjpath += parameters['s3']['objectpath']

        if not (s3.PutDir(set['bucket'], parameters['s3']['srcpath'], destobjpath)):
             raise ValueError('{} {} failed'.format(__file__, __name__))

        with tempfile.TemporaryDirectory() as tmpdirname:
            if not (s3.GetDir(set['bucket'], destobjpath, tmpdirname)):
                raise ValueError('{} {} failed'.format(__file__, __name__))

            infiles = os.listdir(parameters['s3']['srcpath'])
            outfiles = os.listdir(tmpdirname)

            dcmp = dircmp(parameters['s3']['srcpath'], tmpdirname)
            assert(len(dcmp.diff_files)==0)
            

        #  RemoveObjects(self, bucket, setname=None, pattern='**', recursive=False):
        if not s3.RemoveObjects(set['bucket'], destobjpath, recursive=True):
            raise ValueError('{} {} RemoveObjects({}, {}, recursive={}) failed'.format(__file__, __name__, set['bucket'], destobjpath, recursive=True))


    # def CloneObjects(self, destbucket, destsetname, srcS3, srcbucket, srcsetname):
    def test_CloneObjects(self):
        parameters = ReadDict(test_config)
        if 's3' not in parameters:
            raise ValueError('s3 not in {}'.format(test_config))

        # s3Src, _, s3SrcDef = Connect(parameters['s3']['credentials'], s3_name=parameters['s3']['srcS3'])
        # s3Dest, _, s3DestDef = Connect(parameters['s3']['credentials'], s3_name=parameters['s3']['destS3'])

        # srcSet = None
        # if parameters['s3']['srcSet'] in s3SrcDef['sets']:
        #     srcSet = s3SrcDef['sets'][parameters['s3']['srcSet']]
        # else:
        #     raise ValueError('{} {} clone srcSet failure {}'.format(__file__, __name__, parameters['s3']['srcSet']))

        # destSet = None
        # if parameters['s3']['destSet'] in s3DestDef['sets']:
        #     destSet = s3DestDef['sets'][parameters['s3']['destSet']]
        # else:
        #     raise ValueError('{} {} clone destSet failure {}'.format(__file__, __name__, parameters['s3']['destSet']))
        # destpath = ''
        # if type(destSet['prefix']) == str and len(destSet['prefix']) >= 0:
        #     destpath = destSet['prefix']
        # if type(parameters['s3']['dest']) == str and len(parameters['s3']['dest']) >= 0:
        #     if len(destpath) > 0:
        #         destpath += '/'
        #     destpath += parameters['s3']['dest']

        # srcpath = ''
        # if type(srcSet['prefix']) == str and len(srcSet['prefix']) >= 0:
        #     srcpath = srcSet['prefix']
        # if type(parameters['s3']['src']) == str and len(parameters['s3']['src']) >= 0:
        #     if len(srcpath) > 0:
        #         srcpath += '/'
        #     srcpath += parameters['s3']['src']

        # s3Dest.CloneObjects(destSet['bucket'], destpath , s3Src, srcSet['bucket'], srcpath)
        
        s3, _, s3def = Connect(parameters['s3']['credentials'])

        set = None
        if parameters['s3']['testset'] in s3def['sets']:
            set = s3def['sets'][parameters['s3']['testset']]
        else:
            raise ValueError('{} {} clone set undefined {}'.format(__file__, __name__, parameters['s3']['testset']))
        srcobjepath = set['prefix']
        if srcobjepath is not None and len(srcobjepath) > 0:
            srcobjepath += '/'
        srcobjepath += parameters['s3']['objectpath']

        if not (s3.PutDir(set['bucket'], parameters['s3']['srcpath'], srcobjepath)):
            raise ValueError('{} {} PutDir failed'.format(__file__, __name__))

        destobjpath = set['prefix']
        if destobjpath is not None and len(destobjpath) > 0:
            destobjpath += '/'
        destobjpath += parameters['s3']['objectpath'] + '_dest'
        if not s3.CloneObjects(set['bucket'], destobjpath , s3, set['bucket'], srcobjepath):
            raise ValueError('{} {} CloneObjects failed'.format(__file__, __name__))

        #  RemoveObjects(self, bucket, setname=None, pattern='**', recursive=False):
        if not s3.RemoveObjects(set['bucket'], srcobjepath, recursive=True):
            raise ValueError('{} {} RemoveObjects({}, {}, recursive={}) failed'.format(__file__, __name__, set['bucket'], srcobjepath, recursive=True))

        if not s3.RemoveObjects(set['bucket'], destobjpath, recursive=True):
            raise ValueError('{} {} RemoveObjects({}, {}, recursive={}) failed'.format(__file__, __name__, set['bucket'], destobjpath, recursive=True))

if __name__ == '__main__':
    unittest.main()


import sys
import os

from .s3 import *
from .jsonutil import *
from .imutil import ImUtil, ImTransform

def parse_arguments():
    parser = argparse.ArgumentParser(description='Process arguments')

    parser.add_argument('-d', '--debug', action='store_true',help='Wait for debuggee attach')   
    parser.add_argument('-debug_port', type=int, default=3000, help='Debug port')
    parser.add_argument('-debug_address', type=str, default='0.0.0.0', help='Debug port')

    parser.add_argument('-credentials', type=str, default='creds.yaml', help='Credentials file.')

    parser.add_argument('-src', type=str, default=None, help='source path')
    parser.add_argument('-dest', type=str, default=None, help='destination path')

    parser.add_argument('--putdir', '-p', action='store_true',help='Put directory in S3') 
    parser.add_argument('--getdir', '-g', action='store_true',help='Get directory from S3') 
    parser.add_argument('-set', type=str, default='dataset', help='set defined in credentials file')

    parser.add_argument('-clone', action='store_true',help='Clone objects from source to destination S3')
    parser.add_argument('-srcS3', type=str, default=None, help='path to source directory')
    parser.add_argument('-srcSet', type=str, default='dataset', help='set defined in credentials file')
    parser.add_argument('-destS3', type=str, default=None, help='path to source directory')
    parser.add_argument('-destSet', type=str, default='dataset', help='set defined in credentials file')

    parser.add_argument('--version_str', '-v' type=str, default=None, help='version in config file')

    args = parser.parse_args()
    return args

def main(args):
    result = 0
    if args.putdir:
        s3, creds, s3def = Connect(args.credentials)
        if args.set not in s3def['sets']:
            print('putdir failed: args.set {} not found in credentials file'.format(args.set))
        elif args.src is None:
            print('putdir failed: args.src is None')
        elif args.dest is None:
            print('putdir failed: args.dest is None')
        else:
            dest = '{}/{}'.format(s3def['sets'][args.set]['prefix'], args.dest)
            s3.PutDir(s3def['sets'][args.set]['bucket'], args.src, dest)

    if args.getdir:
        s3, creds, s3def = Connect(args.credentials)
        if args.set not in s3def['sets']:
            print('getdir failed: args.set {} not found in credentials file'.format(args.set))
        elif args.src is None:
            print('getdir failed: args.src is None')
        elif args.dest is None:
            print('getdir failed: args.dest is None')
        else:
            src = '{}/{}'.format(s3def['sets'][args.set]['prefix'], args.src)
            s3.GetDir(s3def['sets'][args.set]['bucket'], src, args.dest)

    if args.clone:
        s3Src, _, s3SrcDef = Connect(args.credentials, s3_name=args.srcS3)
        s3Dest, _, s3DestDef = Connect(args.credentials, s3_name=args.destS3)

        srcSet = None
        if args.srcSet in s3SrcDef['sets']:
            srcSet = s3SrcDef['sets'][args.srcSet]
        else:
            raise ValueError('{} {} clone srcSet failure {}'.format(__file__, __name__, args.srcSet))

        destSet = None
        if args.destSet in s3DestDef['sets']:
            destSet = s3DestDef['sets'][args.destSet]
        else:
            raise ValueError('{} {} clone destSet failure {}'.format(__file__, __name__, args.destSet))

        s3Dest.CloneObjects(destSet['bucket'], args.dest , s3Src, srcSet['bucket'], args.src)

    if args.version_str is not None:
        from .version import VersionString
        config = ReadDict(args.version_str)
        result = VersionString(config)

    print('pymluitil complete')
    return result
    
if __name__ == '__main__':
    import argparse
    args = parse_arguments()

    if args.debug:
        print("Wait for debugger attach on {}:{}".format(args.debug_address, args.debug_port))
        import debugpy

        debugpy.listen(address=(args.debug_address, args.debug_port)) # Pause the program until a remote debugger is attached
        debugpy.wait_for_client()  # Pause the program until a remote debugger is attached
        print("Debugger attached")

    result = main(args)